EN, CY = "en", "cy"
SUPPORTED_LANGUAGES = {EN, CY}

NOT_PROVIDED = {
    EN: "Not provided",
    CY: "Heb ei ddarparu",
}

YES = {
    EN: "Yes",
    CY: "Oes",
}

NO = {
    EN: "No",
    CY: "Nac Oes",
}
